// Ternary DAG spellchecker
// (c) Peter Kankowski, 2008. kankowski@narod.ru

#define _CRT_SECURE_NO_DEPRECATE
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <ctype.h>
#include "spellchecker.h"

// ======================
// Tests
// ======================

int g_found;

static void test_enum_proc(const char* found, void* param) {
	const char * expected = (const char*)param;
	if(0 == strcmp(expected, found))
	{
		assert(g_found == 0);
		g_found = 1;
	}
}

static void test_suggestion(ST_FILE_NODE * dict, const char * word, const char * expected)
{
	g_found = 0;
	ST_enum_suggestions(dict, word, test_enum_proc, (void*)expected);
	assert(g_found);
}

static void enum_print(const char* found, void* param) {
	param = param;
	printf("%s\n", found);
}

int main() {
	ST_FILE_NODE * dict = ST_open_dict("spellchk.dat");
	if (!dict) {
		puts("Open dictionary failed\n");
		return 1;
	}
	assert( ST_find_exact(dict, "address") );
	assert( !ST_find_exact(dict, "addres") );
	assert( !ST_find_exact(dict, "ddress") );
	assert( !ST_find_exact(dict, "addressd") );
	assert( ST_find_exact(dict, "you") );
	assert( ST_find_exact(dict, "I") );
	assert( !ST_find_exact(dict, "") );
	assert( ST_find_exact(dict, "Address") );
	assert( ST_find_exact(dict, "ADDRESS") );
	assert( !ST_find_exact(dict, "AddReSS") );
	assert( !ST_find_exact(dict, "ADDReSS") );
	assert( !ST_find_exact(dict, "aDDReSS") );
	assert( ST_find_exact(dict, "Me") );
	assert( ST_find_exact(dict, "ME") );
	assert( !ST_find_exact(dict, "mE") );
	
	test_suggestion(dict, "address", "address");
	test_suggestion(dict, "addres", "address");
	test_suggestion(dict, "berru", "berry");
	test_suggestion(dict, "brotjer", "brother");
	test_suggestion(dict, "sometme", "sometime");
	test_suggestion(dict, "roket", "rocket");
	test_suggestion(dict, "sory", "sorry");
	test_suggestion(dict, "comrpession", "compression");
	test_suggestion(dict, "ghjost", "ghost");
	test_suggestion(dict, "cando", "can do");

	// Print suggestions for the entered words
	printf("Enter text or an empty line to quit:\n");
	int words;
	do {
		char line[4096];
		fgets(line, sizeof(line), stdin);
		
		char * lineend = line + strlen(line);
		char * end = line;
		words = 0;
		do {
			// Skip punctuation, spaces, and digits
			char * word = end;
			while(!isalpha(*word) && *word != '\0')
				word++;
			if (word >= lineend)
				break;
			end = word;
			while(isalpha(*end) || *end == '\'')
				end++;
			*end++ = '\0';
		
			if (ST_find_exact(dict, word))
				printf("%s was found in the dictionary\n", word);
			else {
				printf("Suggestions for %s:\n", word);
				ST_enum_suggestions(dict, word, enum_print, NULL);
				printf("\n");
			}
			words++;
		} while(end < lineend);
	} while(words > 0);

	ST_close_dict(dict);
}
